//
// CMain.cpp
//
// BSP_Engine
//
// Copyright Luk Polok  2000
//

#include <windows.h>
#include <malloc.h>
#include <stdio.h>
#include <math.h>

#include "DirectX.h"
#include "Matrix.h"
#include "Load_3ds.h"
#include "Raster.h"
#include "Main.h"


#define APPNAME "Wire-Frame_Engine"
#define WNDNAME "Wire - Frame Engine 1.0"

#define PI 3.1415926

static unsigned __int32 *video;
static int nWidth = 640;
static int nHeight = 480;
static World *svet;
static Matrix cmatrix;

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR szCmdline, int nCmdShow)
{
	char szAppName[] = APPNAME;
	WNDCLASSEX wndclass;
	HWND hwnd;
	MSG msg;
	int i;

	wndclass.cbSize			= sizeof(wndclass);
	wndclass.style			= CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc	= Wokno;
	wndclass.cbClsExtra		= 0;
	wndclass.cbWndExtra		= 0;
	wndclass.hInstance		= hInstance;
	wndclass.hIcon			= LoadIcon(hInstance, IDI_APPLICATION);
	wndclass.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wndclass.hbrBackground	= (HBRUSH)GetStockObject(WHITE_BRUSH);
	wndclass.lpszMenuName	= 0;
	wndclass.lpszClassName	= szAppName;
	wndclass.hIconSm		= LoadIcon(NULL, IDI_APPLICATION);
	RegisterClassEx(&wndclass);
	hwnd = CreateWindow(szAppName,
		WNDNAME, WS_POPUP,
		CW_USEDEFAULT, CW_USEDEFAULT,
		nWidth, nHeight,
		NULL, NULL,	hInstance, NULL);
	ShowWindow(hwnd, nCmdShow);
	UpdateWindow(hwnd);
	ShowCursor(FALSE);
	// vytvo okno

	Matrix_Init(&cmatrix);
	Matrix_Rotate_Z(&cmatrix, (float)-PI);
	Matrix_Rotate_X(&cmatrix, (float)-PI/6);
	Matrix_Translate(&cmatrix, 0, 0, -150);
	// kamera ok

	if((svet = Load_3DS("svet.3ds")) == NULL)
		return 0;
	for(i = 0; i < svet->n_object_num; i ++)
		Matrix_Init(&svet->object[i].m_object);
	// svet ok

	Init_Raster(nWidth, nHeight);
	// engine ok

	SetTimer(hwnd, 0, 20, NULL);

	InitDirectX(hwnd, nWidth, nHeight);
	// spust directX

	while(GetMessage(&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	TerminateDirectX();

	return msg.wParam;
}

LRESULT CALLBACK Wokno (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
	int i;

	switch(iMsg) {
	case WM_KEYDOWN :
		if(wParam == 27) {
			ShowCursor(TRUE);
			SendMessage(hwnd, WM_CLOSE, 0, 0);
			return 0;
		}
		return 0;

	case WM_TIMER :
		if(InitVideoBuffer(&video)){
			memset((char*)video, 0, sizeof(unsigned __int32) * nWidth * nHeight);

			for(i = 0; i < svet->n_object_num; i ++){
				Matrix_Rotate_Y(&svet->object[i].m_object, (float)PI/50);
				DrawObject(&svet->object[i].m_object,
					&cmatrix, &svet->object[i], video);
			}
		
			FlipVideoBuffer();
		}
		return 0;

	case WM_DESTROY:
		PostQuitMessage (0);
		return 0;
	}
	return DefWindowProc (hwnd, iMsg, wParam, lParam) ;
}





