/*
                                +---------------------------------+
                                |                                 |
                                |     ***   Rasterizer   ***      |
                                |                                 |
                                |  Copyright   -tHE SWINe- 2002  |
                                |                                 |
                                |           Raster.cpp            |
                                |                                 |
                                +---------------------------------+
*/

#include <windows.h>
#include <math.h>
#include "Load_3DS.h"
#include "Matrix.h"
#include "Raster.h"

static int n_Width, n_Height;
static float z_delta;

#define PI 3.141592653579893238462643383279502884

// Nastav parametry rastru (vku a ku)
void Init_Raster(int n_New_Width, int n_New_Height)
{
    n_Width = n_New_Width;
    n_Height = n_New_Height;
	//
	z_delta = z_delta = (float)fabs((float)n_Width / ((float)tan(90 * (PI / 180) / 2) * 2));
	// 90 stup vhled
}

// Kresl zelenou ru
static void inline Line(int xa, int ya, int xb, int yb, unsigned __int32 *vram)
{
    float len, stepx, stepy, x, y;
    int i, xlen, ylen;

    x = (float)xa;
    y = (float)ya;
    xlen = xa - xb;
    ylen = ya - yb;
    len = (float)fabs(((fabs(ylen) > fabs(xlen))? ylen : xlen));
    stepx = xlen / len;
    stepy = ylen / len;

    for(i = 0; i < (int)len; i ++){
        if(x > 0 && x < n_Width && y > 0 && y < n_Height)
            vram[(int)x + n_Width * (int)y] = RGB(150, 255, 100); // MATRIX zelen
        x -= stepx;
        y -= stepy;
    }
}

// Nakresl rov trojhelnk
static void inline Wire_Polygon(Face *fac, unsigned __int32 *vram)
{
    Line((int)fac->vertex[0]->x, (int)fac->vertex[0]->y, 
           (int)fac->vertex[1]->x, (int)fac->vertex[1]->y, vram);
    Line((int)fac->vertex[2]->x, (int)fac->vertex[2]->y, 
           (int)fac->vertex[1]->x, (int)fac->vertex[1]->y, vram);
    Line((int)fac->vertex[0]->x, (int)fac->vertex[0]->y, 
           (int)fac->vertex[2]->x, (int)fac->vertex[2]->y, vram);
}

// nakresl objekct p_object s matic m_object
// z pohledu kamery s matic m_camera do pamti vram
void DrawObject(Matrix *m_object, Matrix *m_camera,
        Object *p_object, unsigned __int32 *vram)
{
    float x, y, z, tx, ty, tz;
    Vertex vtx[3];
    Matrix m, im;
    int i, j;
    Face fac;
    
    fac.vertex[0] = &vtx[0];
    fac.vertex[1] = &vtx[1];
    fac.vertex[2] = &vtx[2];
    // vertexy ve faci

    Matrix_Inverse(m_camera, &m);
    Matrix_Multiply(&m, m_object, &m);
    Matrix_Inverse(&m, &im);
    // transformacni matice

    for(i = 0; i < p_object->n_face_num; i ++) {
        if(im[3][0] * p_object->face[i].normal.a + im[3][1] * p_object->face[i].normal.b +
           im[3][2] * p_object->face[i].normal.c + p_object->face[i].normal.d > 0.1)
            continue;
        // backface culling

        for(j = 0; j < 3; j ++) {
            x = p_object->face[i].vertex[j]->x;
            y = p_object->face[i].vertex[j]->y;
            z = p_object->face[i].vertex[j]->z;

            tx = x * m[0][0] + y * m[1][0] + 
                 z * m[2][0] +     m[3][0];
            ty = x * m[0][1] + y * m[1][1] + 
                 z * m[2][1] +     m[3][1];
            tz = x * m[0][2] + y * m[1][2] + 
                 z * m[2][2] +     m[3][2];
            // transformace

            if(tz > 0.1){
                tx = n_Width / 2 + (tx * z_delta) / tz;
                ty = n_Height / 2 + (ty * z_delta) / tz;
                fac.vertex[j]->x = tx;
                fac.vertex[j]->y = ty;
                fac.vertex[j]->z = tz;
            }
            // perspektivni korekce
        }
        
        Wire_Polygon(&fac, vram);
    }
}
